import * as i0 from '@angular/core';
import { Injectable, Inject, InjectionToken, NgModule } from '@angular/core';
import * as i2 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i1 from '@ng-bootstrap/ng-bootstrap';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import * as i1$1 from '@angular/router';
import { NavigationEnd } from '@angular/router';
import { Subject, merge, take, filter, map, tap } from 'rxjs';

const MODAL_NAME_PARAM = 'routableModal';
const PREFIX_MODAL_PARAM = 'rmp_';

class RoutableModalService {
    constructor(modalSvc, location, config) {
        this.modalSvc = modalSvc;
        this.location = location;
        this.config = config;
        this.openModal = new Subject();
        this.openModal$ = this.openModal.asObservable();
    }
    show(modalName, options, params) {
        var _a;
        const routableModalConf = (_a = this.config) === null || _a === void 0 ? void 0 : _a.modalsConfigurations.find((c) => c.name == modalName);
        if (!routableModalConf) {
            throw new Error('Modal with name ' +
                modalName +
                ' dont exist. Add to NgBootstraproutableModalModule.forRoot(conf)');
        }
        const newParams = Object.assign({ [MODAL_NAME_PARAM]: routableModalConf.name }, Object.entries(params).reduce((acc, [key, value]) => (Object.assign(Object.assign({}, acc), { [PREFIX_MODAL_PARAM + key]: value })), {}));
        this.updateQueryParamsInternal(newParams);
        // OPEN MODAL
        const modalRef = this.modalSvc.open(routableModalConf.component, options);
        merge(modalRef.dismissed, modalRef.closed)
            .pipe(take(1))
            .subscribe((_) => {
            this.resetParamsInternal();
        });
        this.setOpenModalRef(routableModalConf.name, modalRef);
        return modalRef;
    }
    updateQueryParams(params) {
        const newParams = Object.assign({}, Object.entries(params).reduce((acc, [key, value]) => (Object.assign(Object.assign({}, acc), { [PREFIX_MODAL_PARAM + key]: value })), {}));
        this.updateQueryParamsInternal(newParams);
    }
    setOpenModalRef(name, ref) {
        this.openModal.next({ name, ref });
    }
    resetParamsInternal() {
        this.openModal.next(null);
        const currentUrl = this.location.path();
        const urlTree = this.location.normalize(this.location.prepareExternalUrl(currentUrl));
        const queryParams = new URLSearchParams(urlTree);
        for (const [key, value] of Array.from(queryParams)) {
            if (key == MODAL_NAME_PARAM) {
                queryParams.delete(key);
            }
            if (key.startsWith(PREFIX_MODAL_PARAM)) {
                queryParams.delete(key);
            }
        }
        const newQueryParams = queryParams.toString();
        const baseUrl = urlTree.split('?')[0];
        const newUrl = newQueryParams === '' ? baseUrl : `${baseUrl}?${newQueryParams}`;
        this.location.replaceState(newUrl);
    }
    updateQueryParamsInternal(params) {
        const currentUrl = this.location.path();
        const urlTree = this.location.normalize(this.location.prepareExternalUrl(currentUrl));
        const queryParams = new URLSearchParams(urlTree);
        for (const [key, value] of Object.entries(params)) {
            queryParams.set(key, value);
        }
        const newQueryParams = queryParams.toString();
        const newUrl = `${urlTree.split('?')[0]}?${newQueryParams}`;
        this.location.replaceState(newUrl);
    }
}
RoutableModalService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: RoutableModalService, deps: [{ token: i1.NgbModal }, { token: i2.Location }, { token: ROUTABLE_MODAL_CONFIG }], target: i0.ɵɵFactoryTarget.Injectable });
RoutableModalService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: RoutableModalService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: RoutableModalService, decorators: [{
            type: Injectable
        }], ctorParameters: function () {
        return [{ type: i1.NgbModal }, { type: i2.Location }, { type: undefined, decorators: [{
                        type: Inject,
                        args: [ROUTABLE_MODAL_CONFIG]
                    }] }];
    } });

const ROUTABLE_MODAL_CONFIG = new InjectionToken('ROUTABLE_MODAL_CONFIG');
class RoutingInterceptorService {
    constructor(router, modalService, routableModalService, config) {
        this.router = router;
        this.modalService = modalService;
        this.routableModalService = routableModalService;
        this.config = config;
    }
    watchRouteChanges() {
        return this.router.events.pipe(filter((event) => event instanceof NavigationEnd), map((event) => this.getQueryParams(event.url)), tap((data) => {
            var _a;
            const modal = (_a = this.config) === null || _a === void 0 ? void 0 : _a.modalsConfigurations.find((c) => c.name == data.name);
            if (modal) {
                const modalRef = this.modalService.open(modal.component);
                modalRef.componentInstance.params = data.parmas;
                this.routableModalService.setOpenModalRef(data.name, modalRef);
                merge(modalRef.dismissed, modalRef.closed)
                    .pipe(take(1))
                    .subscribe((_) => {
                    this.routableModalService.resetParamsInternal();
                });
            }
        }));
    }
    // UTILS
    getQueryParams(url) {
        var _a;
        const [baseUrl, queryString] = url.split('?');
        if (queryString) {
            const kv = queryString.split('&').map((p) => p.split('='));
            const modalName = (_a = kv.find(([key, value]) => key == MODAL_NAME_PARAM)) === null || _a === void 0 ? void 0 : _a[1];
            const parmas = kv
                .filter(([key, value]) => key.startsWith(PREFIX_MODAL_PARAM))
                .reduce((acc, [key, value]) => (Object.assign(Object.assign({}, acc), { [key.split(PREFIX_MODAL_PARAM)[1]]: value })), {});
            return { name: modalName, parmas, c: this.config };
        }
        return {};
    }
}
RoutingInterceptorService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: RoutingInterceptorService, deps: [{ token: i1$1.Router }, { token: i1.NgbModal }, { token: RoutableModalService }, { token: ROUTABLE_MODAL_CONFIG }], target: i0.ɵɵFactoryTarget.Injectable });
RoutingInterceptorService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: RoutingInterceptorService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: RoutingInterceptorService, decorators: [{
            type: Injectable
        }], ctorParameters: function () {
        return [{ type: i1$1.Router }, { type: i1.NgbModal }, { type: RoutableModalService }, { type: undefined, decorators: [{
                        type: Inject,
                        args: [ROUTABLE_MODAL_CONFIG]
                    }] }];
    } });

class NgBootstraproutableModalModule {
    constructor(routingInterceptorService) {
        this.routingInterceptorService = routingInterceptorService;
        this.routingInterceptorService
            .watchRouteChanges()
            .subscribe((e) => console.dir(e));
    }
    static forRoot(config) {
        return {
            ngModule: NgBootstraproutableModalModule,
            providers: [
                RoutingInterceptorService,
                RoutableModalService,
                {
                    provide: ROUTABLE_MODAL_CONFIG,
                    useValue: config,
                },
            ],
        };
    }
}
NgBootstraproutableModalModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: NgBootstraproutableModalModule, deps: [{ token: RoutingInterceptorService }], target: i0.ɵɵFactoryTarget.NgModule });
NgBootstraproutableModalModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.3.0", ngImport: i0, type: NgBootstraproutableModalModule, imports: [CommonModule, NgbModule], exports: [NgbModule] });
NgBootstraproutableModalModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: NgBootstraproutableModalModule, imports: [CommonModule, NgbModule, NgbModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.3.0", ngImport: i0, type: NgBootstraproutableModalModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [CommonModule, NgbModule],
                    exports: [NgbModule],
                }]
        }], ctorParameters: function () { return [{ type: RoutingInterceptorService }]; } });

/*
 * Public API Surface of ng-bootstrap-routable-modal
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgBootstraproutableModalModule, RoutableModalService };
//# sourceMappingURL=ng-bootstrap-routable-modal.mjs.map
